//
//  s95160_s1.h	  Segment buffer support Layer1 (Primitive segment, horizontal relation)
//  Graphic LCD module "SUCCESS ELECTRONIC S95160" I/O library
//  Copyright(C)2013-2014 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2013.12.20 v0.0  First cording
//    2013.12.27 v0.1  Release
//    2013.12.28 v0.1a Reducing the arguments
//    2013.12.30 v0.1b Scratch and build, value base -> refer base arguments
//    2014.01.02 v0.1b Support the Gage, 7segLED figure
//    2014.01.15 v0.1b Support the Bar chart figure
//

#ifndef S95160_S1_H_
#define S95160_S1_H_

#include <stddef.h>						// size_t
#include "s95160_2.h"					// COLOR

//
//	Configuration parameter
//
#define LCD_SEGMENT_STRING_COLUMN_MAX		31					// Temporary buffer size

//
//	Basic type
//
struct lcd_segmentCoordinate {			// Screen point
	short			x;
	short			y;
};
struct lcd_segmentArea {				// Screen rect
	short			x;
	short			y;
	short			width;
	short			height;
};
struct lcd_segmentColors {				//
	unsigned short	face;
	unsigned short	back;
};

//
//	Control block
//

// body = figure
struct lcd_segmentFigure {			// Minimum common structure
	struct lcd_segment				*seg;
	struct lcd_segmentFigFlag {
		unsigned int					dirty:	1;	// 1:Need a redisplay
		unsigned int					hidden:	1;	// 1:Disable entry
		unsigned int					malloc: 1;	// 0:fig="Static memory area" 1:fig="Heap memory area"
		unsigned int					init: 	1;	// 1:Request for initial display (option)
	}								flags;
};
struct lcd_segmentRect {			// Figure member
	struct lcd_segmentFigure		fig;			// Essential item
	struct lcd_segmentArea			area;
	unsigned short					color;
};
struct lcd_segmentPoly {			// Figure member
	struct lcd_segmentFigure		fig;			// Essential item
	unsigned short					color;
	short 							lineSize;
	int 							n;
	struct lcd_segmentCoordinate	*point;
};
struct lcd_segmentStr {				// Figure member, Single line simple string figure
	struct lcd_segmentFigure		fig;			// Essential item
	struct lcd_segmentCoordinate	point;
	struct lcd_segmentColors		color;
	char							*str;
};
struct lcd_segmentImage {			// Figure member, Bit image figure
	struct lcd_segmentFigure		fig;			// Essential item
	struct lcd_segmentArea			area;
	unsigned short					*image;
};
struct lcd_segmentText {			// Figure member, Text string figure
	struct lcd_segmentFigure		fig;			// Essential item
	struct lcd_segmentArea			area;
	struct lcd_segmentColors		color;
	struct lcd_segmenTextAttr {
		unsigned int				horizontalAlign: 2;
		unsigned int				varticalAlign: 2;
	}								attr;
	char							*str;
};
#define LCD_SEGMENT_TEXT_ALIGN_LEFT			0
#define LCD_SEGMENT_TEXT_ALIGN_UP			0
#define LCD_SEGMENT_TEXT_ALIGN_CENTER		1
#define LCD_SEGMENT_TEXT_ALIGN_RIGHT		2
#define LCD_SEGMENT_TEXT_ALIGN_DOWN			2

struct lcd_segmentGage {			// Figure member, Simple bar graph
	struct lcd_segmentFigure		fig;			// Essential item
	struct lcd_segmentArea			area;
	struct lcd_segmentColors		color;
	struct lcd_segmenGageAttr {
		unsigned int					vartical:		1;
	}								attr;
	float							value;			// 0.0 - 1.0
	struct {
		int								beforePos;
	}								internal;
};

struct lcd_segmentBarChart {		// Figure member
	struct lcd_segmentFigure		fig;			// Essential item
	struct lcd_segmentArea			area;
	unsigned short					backColor;
	struct lcd_segmenGageAttr 		attr;
	int								number;
	float							*value;			// 0.0 - 1.0
	unsigned short					*color;
};

struct lcd_segment7seg {			//  Figure member, 7segment LED
	struct lcd_segmentFigure		fig;			// Essential item
	struct lcd_segmentArea			area;
	struct lcd_segmentColors		color;
	short							lineWidth;
	int								column;
	int								dotPoint;
	int								value;
};

// header = segment
struct lcd_segment {
	struct lcd_segment		*next;			// Horizontal chain link
	struct lcd_segment		*prev;			// Backward relation
	void					*link;			// fig/seg; 1st entry is a group attribute, last entry is a parent link, Other is a display figure data
	int						(*dispProg)(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *base);	// = fig types
};

struct lcd_segmentGlobalFlags {
	unsigned int	dirty:		1;
	unsigned int	reserve:	15;
};
extern struct lcd_segmentGlobalFlags	lcd_segmentGlobalFlags;


//
//	API
//

// Alloc
extern struct lcd_segmentFigure *	lcd_segmentAllocFigure(size_t len);
//extern struct lcd_segmentRect*	lcd_segmentAllocRect()
//extern struct lcd_segmentPoly*	lcd_segmentAllocPoly()
//extern struct lcd_segmentStr*		lcd_segmentAllocStr()
//extern struct lcd_segmentImage*	lcd_segmentAllocImage()
//extern struct lcd_segmentText*	lcd_segmentAllocText()
//extern struct lcd_segmentGage*	lcd_segmentAllocGage()
//extern struct lcd_segmentBarChart*lcd_segmentAllocBarChart()
//extern struct lcd_segment7seg*	lcd_segmentAlloc7seg()
#define								lcd_segmentAllocRect()						((struct lcd_segmentRect  *)lcd_segmentAllocFigure(sizeof(struct lcd_segmentRect)))
#define								lcd_segmentAllocPoly()						((struct lcd_segmentPoly  *)lcd_segmentAllocFigure(sizeof(struct lcd_segmentPoly)))
#define								lcd_segmentAllocStr()						((struct lcd_segmentStr   *)lcd_segmentAllocFigure(sizeof(struct lcd_segmentStr)))
#define								lcd_segmentAllocImage()						((struct lcd_segmentImage *)lcd_segmentAllocFigure(sizeof(struct lcd_segmentImage)))
#define								lcd_segmentAllocText()						((struct lcd_segmentText  *)lcd_segmentAllocFigure(sizeof(struct lcd_segmentText)))
#define								lcd_segmentAllocGage()						((struct lcd_segmentGage  *)lcd_segmentAllocFigure(sizeof(struct lcd_segmentGage)))
#define								lcd_segmentAllocBarChart()					((struct lcd_segmentBarChart *)lcd_segmentAllocFigure(sizeof(struct lcd_segmentBarChart)))
#define								lcd_segmentAlloc7seg()						((struct lcd_segment7seg  *)lcd_segmentAllocFigure(sizeof(struct lcd_segment7seg)))

// Dealloc
extern void							lcd_segmentDeallocFigure(struct lcd_segmentFigure *fig);

// New
extern struct lcd_segment *			lcd_segmentNewFigure(struct lcd_segmentFigure *brother, struct lcd_segmentFigure *fig, size_t l, int (*dispProg)(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *base));
//extern struct lcd_segment* 		lcd_segmentNewRect( struct lcd_segmentFigure *brother,  struct lcd_segmentRect  *rect);
//extern struct lcd_segment* 		lcd_segmentNewPoly( struct lcd_segmentFigure *brother,  struct lcd_segmentPoly  *poly);
//extern struct lcd_segment*		lcd_segmentNewStr(  struct lcd_segmentFigure *brother,  struct lcd_segmentStr   *str);
//extern struct lcd_segment*		lcd_segmentNewImage(struct lcd_segmentFigure *brother,  struct lcd_segmentImage *image);
//extern struct lcd_segment*		lcd_segmentNewText( struct lcd_segmentFigure *brother,  struct lcd_segmentText  *str);
//extern struct lcd_segment*		lcd_segmentNewGage( struct lcd_segmentFigure *brother,  struct lcd_segmentGage  *gage);
//extern struct lcd_segment*		lcd_segmentNewBarChart(struct lcd_segmentFigure *brother,  struct lcd_segmentBarChart  *barChart);
//extern struct lcd_segment*		lcd_segmentNew7seg( struct lcd_segmentFigure *brother,  struct lcd_segment7seg  *led7seg);
#define								lcd_segmentNewRect( brother, rect)			lcd_segmentNewFigure((brother), (struct lcd_segmentFigure *)(rect),  sizeof(struct lcd_segmentRect),  lcd_segmentDisplayRect)
#define 							lcd_segmentNewPoly( brother, poly)			lcd_segmentNewFigure((brother), (struct lcd_segmentFigure *)(poly),  sizeof(struct lcd_segmentPoly),  lcd_segmentDisplayPoly)
#define 							lcd_segmentNewStr(  brother, str)			lcd_segmentNewFigure((brother), (struct lcd_segmentFigure *)(str),   sizeof(struct lcd_segmentStr),   lcd_segmentDisplayStr)
#define 							lcd_segmentNewImage(brother, image)			lcd_segmentNewFigure((brother), (struct lcd_segmentFigure *)(image), sizeof(struct lcd_segmentImage), lcd_segmentDisplayImage)
#define								lcd_segmentNewText( brother, str)			lcd_segmentNewFigure((brother), (struct lcd_segmentFigure *)(str),   sizeof(struct lcd_segmentText),  lcd_segmentDisplayText)
#define 							lcd_segmentNewGage( brother, gage)			lcd_segmentNewFigure((brother), (struct lcd_segmentFigure *)(gage),  sizeof(struct lcd_segmentGage),  lcd_segmentDisplayGage)
#define 							lcd_segmentNewBarChart(brother, barChart)	lcd_segmentNewFigure((brother), (struct lcd_segmentFigure *)(barChart), sizeof(struct lcd_segmentBarChart), lcd_segmentDisplayBarChart)
#define 							lcd_segmentNew7seg( brother, led)			lcd_segmentNewFigure((brother), (struct lcd_segmentFigure *)(led),   sizeof(struct lcd_segment7seg),  lcd_segmentDisplay7seg)

// Delete
extern void							lcd_segmentDeleteFigure(struct lcd_segmentFigure *fig);

// Set value
extern void							lcd_segmentSetFigureDirty(struct lcd_segmentFigure *fig);
extern void							lcd_segmentSetFigureHidden(struct lcd_segmentFigure *fig, int f);

//
//	Internal routine
//
extern struct lcd_segment*			lcd_segmentAllocSeg(struct lcd_segment *insertThisAfter);
extern void							lcd_segmentDeallocSeg(struct lcd_segment *seg);

extern int							lcd_segmentDisplayRect( struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint);
extern int							lcd_segmentDisplayPoly( struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint);
extern int							lcd_segmentDisplayStr(  struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint);
extern int							lcd_segmentDisplayImage(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint);
extern int							lcd_segmentDisplayText( struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint);
extern int							lcd_segmentDisplayGage( struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint);
extern int							lcd_segmentDisplayBarChart(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint);
extern int							lcd_segmentDisplay7seg( struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint);
extern void							segDump(struct lcd_segment *seg);

#endif /* S95160_S1_H_ */
