//
//  s95160_s2.c	  Segment buffer support Layer2 (Hierarchy segment, Vertical relation, Touch action)
//  Graphic LCD module "SUCCESS ELECTRONIC S95160" I/O library
//  Copyright(C)2013-2014 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2013.12.26 v0.0  First cording
//    2013.12.27 v0.1  Release
//    2013.12.28 v0.1a Reducing the arguments
//    2013.12.31 v0.1b Scratch and build, value base -> refer base arguments
//    2014.01.04 v0.1b Splitting a source code (s2 -> s2+s3)
//
//


#include "s95160_2.h"					// Display driver, Color code etc...
#include "s95160_t2.h"					// Touch screen driver
#include "s95160_s1.h"					// Under layer
#include "s95160_s2.h"					// My definition for external
#include "s95160_s2.h"					// My definition for external

//#define V01B
//#define DEBUG

// Inline function
#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define ABS(a)		(((a) >  0 ) ? (a) : -(a))

#ifdef DEBUG
#include <stdio.h>
void segDump(struct lcd_segment *p);
#endif

//
//	Common area
//
int	displayLimitTick;

//
//	Group segment
//
struct area {
	short	lx;
	short	ly;
	short	hx;
	short	hy;
};
int areaCheck(struct area *a, struct area *b){
	int	flx;
	int	fly;
	int	fhx;
	int	fhy;
	int	f;

	if (a->lx < b->lx) {
		flx = 0;
	} else if(a->lx < b->hx) {
		flx = 1;
	} else {
		flx = 2;
	}
	if (a->ly < b->ly) {
		fly = 0;
	} else if(a->ly < b->hy) {
		fly = 1;
	} else {
		fly = 2;
	}
	if (a->hx < b->lx) {
		fhx = 0;
	} else if(a->hx < b->hx) {
		fhx = 1;
	} else {
		fhx = 2;
	}
	if (a->hy < b->ly) {
		fhy = 0;
	} else if(a->hy < b->hy) {
		fhy = 1;
	} else {
		fhy = 2;
	}
	f = 1;
	if ((flx == fhx) && (flx != 1)) {
		f = 0;
	}
	if ((fly == fhy) && (fly != 1)) {
		f = 0;
	}
	return(f);
}
#ifdef V01B
int lcd_segmentDisplayGroup(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint){
	int		f;

	f = 0;
	if (!fig->flags.hidden) {
		struct lcd_segmentGroup			*group;
		struct lcd_segment				*groups;
		struct area						a;
		struct lcd_segmentCoordinate	point;
		struct lcd_segment				*figures;

		group   = (struct lcd_segmentGroup *)fig;
		groups  = lcd_segmentGetGroupToSegment(group);
		a.lx    = 32767;
		a.ly    = 32767;
		a.hx    = 0;
		a.hy    = 0;
		point.x = cardinalPoint->x + group->area.x;
		point.y = cardinalPoint->y + group->area.y;

		for(figures = groups->link; figures->next != NULL; figures = figures->next);	// Last entry

		if (fig->flags.dirty) {
			for(figures = figures->prev; figures->prev != NULL; figures = figures->prev) {	// OLD -> NEW
				struct lcd_segmentFigure	*figure;

				figure = (struct lcd_segmentFigure *)lcd_segmentGetGroupFromSegment(figures);
				if (!figure->flags.hidden) {
					if (figures->dispProg) {
						figure->flags.dirty = 1;
						figures->dispProg(figure, &point);
					}
					figure->flags.dirty = 0;
				}
			}
		} else {
			f = 0;
			for(figures = figures->prev; figures != NULL; figures = figures->prev) {	// OLD -> NEW
				struct lcd_segmentFigure	*figure;

				figure = (struct lcd_segmentFigure *)lcd_segmentGetGroupFromSegment(figures);
				if ((!figure->flags.hidden) && (figures->dispProg)) {

					if (figures->dispProg == lcd_segmentDisplayGroup) {
						struct lcd_segmentGroup	*g;

						g = (struct lcd_segmentGroup *)figure;
						if (f) {
							struct area		b;

							b.lx = g->area.x;
							b.ly = g->area.y;
							b.hx = g->area.x + g->area.width;
							b.hy = g->area.y + g->area.height;
							if (areaCheck(&a, &b)) {
								figures->dispProg(figure, &point);
								a.lx = MIN(a.lx, b.lx);
								a.ly = MIN(a.ly, b.ly);
								a.hx = MAX(a.hx, b.hx);
								a.hy = MAX(a.hy, b.hy);
							}
						} else {
							if (figures->dispProg(figure, &point)) {
								f = 1;
								a.lx = MIN(a.lx, g->area.x);
								a.ly = MIN(a.ly, g->area.y);
								a.hx = MAX(a.hx, g->area.x + g->area.width);
								a.hy = MAX(a.hy, g->area.y + g->area.height);
							}
						}
					} else {
						if (f) {
							figures->dispProg(figure, &point);
						} else {
							if (figure->flags.dirty){
								if (figures->dispProg(figure, &point)) {
									f = 1;
								}
							}
						}
					}
					figure->flags.dirty = 0;
				}
			}
		}
		group->fig.flags.dirty = 0;
	}
	return(f);
}

#else

int lcd_segmentDisplayGroup(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint){
	int		f;

	f = 0;
	if (!(fig->flags.hidden) && fig->flags.dirty) {
		struct lcd_segmentGroup			*group;
		struct lcd_segment				*groups;
		struct lcd_segmentCoordinate	point;
		struct lcd_segment				*figures;
		struct lcd_segmentFigure		*figure;

		group   = (struct lcd_segmentGroup *)fig;
		groups  = lcd_segmentGetGroupToSegment(group);
		point.x = cardinalPoint->x + group->area.x;
		point.y = cardinalPoint->y + group->area.y;
		for(figures = groups->link; figures->next != NULL; figures = figures->next);	// Last entry
		for(figures = figures->prev; figures->prev != NULL; figures = figures->prev) {	// OLD -> NEW
			figure = (struct lcd_segmentFigure *)lcd_segmentGetGroupFromSegment(figures);
			if (!(figure->flags.hidden) && figures->dispProg) {
				if (figure->flags.dirty) {
					figures->dispProg(figure, &point);
					figure->flags.dirty = 0;
					f = 1;
				}
			}
		}
		group->fig.flags.dirty = 0;
	}
	return(f);
}

int lcd_segmentSetGroupDirtyWalk(struct lcd_segmentGroup *group){
	int		f;

	f = 0;
	if (!group->fig.flags.hidden) {
		struct lcd_segment			*groups;
		struct lcd_segment			*figures;
		struct lcd_segmentGroup		*figure;

		groups  = lcd_segmentGetGroupToSegment(group);
		for(figures = groups->link; figures->next != NULL; figures = figures->next);	// Last entry

		for(figures = figures->prev; figures != NULL; figures = figures->prev) {		// OLD -> NEW
			figure = lcd_segmentGetGroupFromSegment(figures);

			if ((!figure->fig.flags.hidden) && (figures->dispProg)) {

				if (group->fig.flags.dirty) {
					f = 1;		// Marking to all and low order layer
					figure->fig.flags.dirty = 1;
					figure->fig.flags.init  = 1;
					if (figures->dispProg == lcd_segmentDisplayGroup) {
						lcd_segmentSetGroupDirtyWalk(figure);
					}
				} else {		// Marking to newer figures
					if (figures->dispProg == lcd_segmentDisplayGroup) {
						if (lcd_segmentSetGroupDirtyWalk(figure)) {
							f = 1;
						}
					} else {
						if (figure->fig.flags.dirty) {
							f = 1;
						} else if (f) {
							figure->fig.flags.dirty = 1;
						}
					}
				}
			}
		}
		if (f) {
			group->fig.flags.dirty = 1;
		}
	}
	return(f);
}

#endif

struct lcd_segment *lcd_segmentNewGroup(struct lcd_segmentGroup *parent, struct lcd_segmentGroup *group){
	struct lcd_segment		*my;
	struct lcd_segment		*top;
	struct lcd_segment		*last;

	my   = lcd_segmentNewFigure((struct lcd_segmentFigure *)parent, (struct lcd_segmentFigure *)group, sizeof(struct lcd_segmentGroup), lcd_segmentDisplayGroup);
	top  = lcd_segmentAllocSeg(NULL);		// New independent group seed
	last = lcd_segmentAllocSeg(top);
	if (my && top && last) {
		top->link      = my->link;			// Insert to vertical link
		my->link       = top;				// Move the group attribute
		group->fig.seg = top;				//
		last->dispProg = lcd_segmentDisplayGroup;
		if (parent) {
			last->link = parent->fig.seg;	// Set to parent link
		}
	} else {
		if (last) {
			lcd_segmentDeallocSeg(last);
		}
		if (top) {
			lcd_segmentDeallocSeg(top);
		}
		if (my) {
			lcd_segmentDeallocFigure((struct lcd_segmentFigure *)group);
		}
		my = NULL;
	}
	return(my);
}

struct lcd_segment *lcd_segmentGetGroupFigureSegment(struct lcd_segment *seg) {
	if (seg) {
		while(seg->dispProg == lcd_segmentDisplayGroup) {
			seg = seg->link;
		}
	}
	return(seg);
}

struct lcd_segmentGroup *lcd_segmentGetGroupFromSegment(struct lcd_segment *seg) {
	struct lcd_segmentGroup *fig;

	fig = NULL;
	seg = lcd_segmentGetGroupFigureSegment(seg);
	if (seg) {
		fig = seg->link;
	}
	return(fig);
}

struct lcd_segment *lcd_segmentGetGroupToSegment(struct lcd_segmentGroup *fig) {
	struct lcd_segment *seg;

	seg = NULL;
	if (fig) {
		seg = lcd_segmentGetGroupFigureSegment(fig->fig.seg);
	}
	return(seg);
}

struct lcd_segmentGroup *lcd_segmentGetGroupParent(struct lcd_segmentGroup *fig){
	struct lcd_segmentGroup *group;
	struct lcd_segment 		*seg;

	group = NULL;
	if (fig->fig.seg != NULL) {
		for(seg = fig->fig.seg; seg->next != NULL; seg = seg->next);	// Last segment is a parent group
		group = lcd_segmentGetGroupFromSegment(seg);
	}
	return(group);
}

void lcd_segmentDeleteGroup(struct lcd_segmentGroup *target){
	if (target != NULL) {
		struct lcd_segmentGroup	*parent;
		struct lcd_segment		*parents;
		struct lcd_segment		*targets;

		parent  = lcd_segmentGetGroupParent(target);
		parents = lcd_segmentGetGroupToSegment(parent);
		targets = lcd_segmentGetGroupToSegment(target);

		// Figure delte
		if (targets->prev != NULL) {
			lcd_segmentDeleteFigure((struct lcd_segmentFigure *)target);

		// Group delete
		} else {
			struct lcd_segment		*heads;
			struct lcd_segment 		*s;
			struct lcd_segment 		*tops;
			struct lcd_segment 		*lasts;


			// find a head in parent
			heads = NULL;
			if (parents) {
				for(heads = parents->next; heads != NULL; heads = heads->next) {
					if (heads->link == targets) {
						break;
					}
				}
			}

			// delete all entry
			tops = targets;
			for(s = tops->next; s->next != NULL; ) {
				struct lcd_segment 		*n;

				n = s->next;			// save
				lcd_segmentDeleteGroup(lcd_segmentGetGroupFromSegment(s));
				s = n;					// restore
			}
			lasts = s;

			// delete group elements
			lcd_segmentDeallocSeg(lasts);
			lcd_segmentDeallocSeg(tops);
			lcd_segmentDeallocSeg(heads);
			lcd_segmentDeallocFigure((struct lcd_segmentFigure *)target);
		}

		// Display refresh
		if (parent) {
			lcd_segmentSetGroupDirty(parent);
		}
	}
	return;
}

void lcd_segmentSetGroupHidden(struct lcd_segmentGroup *fig, int f) {
	if (fig) {
		if (fig->fig.flags.hidden != f) {
			lcd_segmentSetFigureHidden((struct lcd_segmentFigure *)fig, f);
			if (f) {	// display -> erase
				struct lcd_segmentGroup *p;

				p = lcd_segmentGetGroupParent(fig);
				if (p) {
					lcd_segmentSetGroupDirty(p);
				}
			}
		}
	}
	return;
}

struct lcd_segmentGroup *lcd_segmentFindGroupByPoint(struct lcd_segmentGroup *group, struct lcd_segmentCoordinate *point) {
	struct lcd_segmentGroup	*found;
	struct lcd_segment		*groups;

	found = NULL;
	groups = lcd_segmentGetGroupToSegment(group);
	if ((groups->prev == NULL) &&		// Group segment
	    (!group->fig.flags.hidden)) {

		if ((group->area.x <= point->x) && (point->x < (group->area.x + group->area.width)) &&
		    (group->area.y <= point->y) && (point->y < (group->area.y + group->area.height))) {
			struct lcd_segmentCoordinate	a;
			struct lcd_segment				*s;

			a.x = point->x - group->area.x;
			a.y = point->y - group->area.y;
			s = groups->link;
			for(s = s->next; s->next != NULL; s = s->next) {
				if (s->dispProg == lcd_segmentDisplayGroup) {
					if ((found = lcd_segmentFindGroupByPoint(lcd_segmentGetGroupFromSegment(s), &a)) != NULL) {
						break;
					}
				}
			}
			if (found == NULL) {
				found = group;
			}
		}
	}
	return(found);
}


//
//	Touch sense
//
void lcd_segmentHandlerTouch(struct lcd_segmentGroup *root){
	static struct lcd_segmentGroup		*group;
	static int							count;
	struct lcd_segmentCoordinate		point;
	short								z;

	z = lcd_touchPoint(&point.x, &point.y);
	if (z > LCD_SEGMENT_TOUCH_THRESHOLD) {
#ifdef DEBUG
		lcd_drawRect(point.x, point.y, 5, 5, LCD_COLOR_PINK);		// DEBUG
#endif	//DEBUG
		count++;
		if (count > LCD_SEGMENT_TOUCH_CHATTER_CANCEL) {
			// Touch "ON"
			if (group == NULL) {
				// Touch in event (one time)
				group = lcd_segmentFindGroupByPoint(root, &point);	// Search target figure with user program
				while(group != NULL) {
					if (group->touchInProg || group->touchOutProg) {
						break;
					}
					group = lcd_segmentGetGroupParent(group);
				}
				if (group) {
					if (group->touchInProg) {
						static struct lcd_segmentGroup	*r;

						for(r = group; r != NULL; r = lcd_segmentGetGroupParent(r)) {
							point.x -= r->area.x;
							point.y -= r->area.y;
						}
						group->touchInProg(group, &point);
					}
				}
			} else {
				// Touch on keep
			}
		}
	} else {
		// Touch "OFF"
		if (group) {
			// Touch out event (one time)
			if (group->touchOutProg) {
				group->touchOutProg(group);
			}
			group = NULL;
		} else {
			// Touch off keep
		}
		count = 0;
	}

	return;
}


//
//	Task handler
//
#ifdef V01B
void lcd_segmentHandler(struct lcd_segmentGroup *root, struct lcd_segmentCoordinate *point){
	if (lcd_segmentGlobalFlags.dirty) {
		if (!lcd_segmentDisplayGroup((struct lcd_segmentFigure *)root, point)) {
			lcd_segmentGlobalFlags.dirty = 0;
		}
	}
	lcd_segmentHandlerTouch(root);
	return;
}
#else

void lcd_segmentHandler(struct lcd_segmentGroup *root, struct lcd_segmentCoordinate *point){
	static int	state;

	switch(state) {
	case 0:
		if (lcd_segmentGlobalFlags.dirty) {
			lcd_segmentSetGroupDirtyWalk(root);
			lcd_segmentGlobalFlags.dirty = 0;
			state = 1;
		} else {
			state = 2;
		}
		break;
	case 1:
		lcd_segmentDisplayGroup((struct lcd_segmentFigure *)root, point);
		state = 0;
		break;
	case 2:
		lcd_segmentHandlerTouch(root);
		state = 0;
		break;
	default:
		break;
	}

	return;
}
#endif

//
//	DEBUG
//
#ifdef DEBUG
#include <stdio.h>						// DEBUG
void segDump(struct lcd_segment *p){
	static int	cnt = 10;

	printf("segDump >>\n");
	for(; p != NULL; p = p->next) {
		char		*pg;

		// print
		if (p->dispProg == lcd_segmentDisplayGroup)			pg="grp";
		else if (p->dispProg == lcd_segmentDisplayRect)		pg="rec";
		else if (p->dispProg == lcd_segmentDisplayPoly)		pg="pol";
		else if (p->dispProg == lcd_segmentDisplayStr)		pg="str";
		else if (p->dispProg == lcd_segmentDisplayImage)	pg="img";
		else if (p->dispProg == lcd_segmentDisplayText)		pg="tex";
		else if (p->dispProg == lcd_segmentDisplayGage)		pg="gag";
		else if (p->dispProg == lcd_segmentDisplay7seg)		pg="7se";
		else if (p->dispProg == NULL)						pg="";
		else												pg="?";
		printf("%8x:%8x %8x %s %8x\n", p, p->next, p->prev, pg, p->link);

		// segment link check
		if (p->next) {
			if (p->next->prev != p) {
				printf("Broken next link\n");
				if (--cnt < 0) {
					for(;;);
				}
			}
		}
		if (p->prev) {
			if (p->prev->next != p) {
				printf("Broken prev link\n");
				if (--cnt < 0) {
					for(;;);
				}
			}
		}
		if (p->dispProg != lcd_segmentDisplayGroup)  {
			struct lcd_segmentFigure *s;

			s = p->link;
			if (s->seg != p) {
				printf("Broken fig back\n");
				if (--cnt < 0) {
					for(;;);
				}
			}
			if (p->next == NULL) {
				printf("last is not parent link (but root is ok)\n");
				if (--cnt < 0) {
					for(;;);
				}
			}
		} else {	// Group in
			if (p->next != NULL){
				struct lcd_segment *s;

				s = p->link;
				if (s->dispProg == lcd_segmentDisplayGroup) {
					printf("link is not group figure\n");
					if (--cnt < 0) {
						for(;;);
					}
				}
				if (s->prev != NULL) {
					printf("link is not group head\n");
					if (--cnt < 0) {
						for(;;);
					}
				}

				segDump(p->link);

			} else {				// last
				struct lcd_segment *s;
				struct lcd_segment *t;
				struct lcd_segment *u;

				s = p->link;
				if (s) {
					if (s->prev != NULL) {
						printf("lasts link is not group head\n");
						if (--cnt < 0) {
							for(;;);
						}
					}

					for(t = p; t->prev != NULL; t = t->prev);
					for(u = s; u != NULL; u = u->next){
						if (u->link == t) {
							break;
						}
					}
					if (u == NULL) {
						printf("lasts link is not chain back\n");
						if (--cnt < 0) {
							for(;;);
						}
					}
				} else {
					printf("found root parent\n");
				}
			}
		}
	}
	printf("<<\n");
}
#endif	// DEBUG

