//
//  s95160_s3.h	  Segment buffer support Layer3 (Composite segment)
//  Graphic LCD module "SUCCESS ELECTRONIC S95160" I/O library
//  Copyright(C)2013-2014 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2014.01.04 v0.1b New, Splitting a source code (s2 -> s2+s3)
//
#ifndef S95160_S3_H_
#define S95160_S3_H_

#include "s95160_s2.h"					//

//
//	Configuration parameter
//
#define KEYPAD_SIZE_W			240
#define KEYPAD_SIZE_H			100
#define TENKEYPAD_SIZE_W		150
#define TENKEYPAD_SIZE_H		150
#define PULLDOWN_SIZE_W_MAX		100
#define PULLDOWN_SIZE_W_MIN		40
#define PULLDOWN_SIZE_H			20

//
//	Control block
//
struct lcd_segmentButton {
	struct lcd_segmentGroup 		grp;			// Essential item
	struct lcd_segmentColors 		color;
	char							*caption;
	struct lcd_segmenButtonAttr {
		unsigned int				state:			1;
		unsigned int				alternateMode:	1;
	}								attr;
	void							(*actionProg)(struct lcd_segmentButton *s);
	struct {
		struct lcd_segmentRect			indicator;
		struct lcd_segmentCoordinate	frame[5];
		struct lcd_segmentPoly			edge1;
		struct lcd_segmentPoly			edge2;
		struct lcd_segmentText			text;
	}								internal;
};

struct lcd_segmentKeyPad {
	struct lcd_segmentGroup 		grp;			// Essential item
	struct lcd_segmentColors 		color;
	void							*targetField;
	struct {
		struct lcd_segmentButton		keyButton[4][10];
		struct lcd_segmentButton		keyButton2[5];
	}								internal;
};

#define TENKEY_NUM_X	4
#define TENKEY_NUM_Y	4
struct lcd_segmentTenKeyPad {
	struct lcd_segmentGroup 		grp;			// Essential item
	struct lcd_segmentColors 		color;
	void							*targetField;
	struct {
		struct lcd_segmentButton		keyButton[TENKEY_NUM_Y][TENKEY_NUM_X];
		int								fp;
	}								internal;
};

struct lcd_segmentSelectionList {
	struct lcd_segmentGroup 		grp;			// Essential item
	struct lcd_segmentColors 		color;
	void							*targetField;
	char							**label;
	int								number;
	struct {
		struct lcd_segmentButton		label[0];
	}								internal;
};

struct lcd_segmentTextField {
	struct lcd_segmentGroup 		grp;			// Essential item
	struct lcd_segmentColors 		color;
	struct lcd_segmenTextAttr		attr;
	char							*str;
	size_t							len;
	void							(*actionProg)(struct lcd_segmentTextField *s);
	struct {
		struct lcd_segmentCoordinate	frame[3];
		struct lcd_segmentPoly			edge;
		struct lcd_segmentText			text;
	}								internal;
};

struct lcd_segmentIntegerField {
	struct lcd_segmentGroup 		grp;			// Essential item
	struct lcd_segmentColors 		color;
	char							*format;
	int								min;
	int								max;
	int								value;
	void							(*actionProg)(struct lcd_segmentIntegerField *s);
	struct {
		struct lcd_segmentCoordinate	frame[3];
		struct lcd_segmentPoly			edge;
		struct lcd_segmentText			text;
		char							buf[16];
	}								internal;
};

struct lcd_segmentFloatField {
	struct lcd_segmentGroup 		grp;			// Essential item
	struct lcd_segmentColors 		color;
	char							*format;
	float							min;
	float							max;
	float							value;
	void							(*actionProg)(struct lcd_segmentFloatField *s);
	struct {
		struct lcd_segmentCoordinate	frame[3];
		struct lcd_segmentPoly			edge;
		struct lcd_segmentText			text;
		char							buf[16];
	}								internal;
};

struct lcd_segmentPullDownField {
	struct lcd_segmentGroup 		grp;			// Essential item
	struct lcd_segmentColors 		color;
	char							**label;
	int								number;
	int								select;
	void							(*actionProg)(struct lcd_segmentPullDownField *s);
	struct {
		struct lcd_segmentCoordinate	frame[3];
		struct lcd_segmentPoly			edge;
		struct lcd_segmentText			text;
	}								internal;
};


//
//	API
//
#define /*    lcd_segmentButton* */	lcd_segmentAllocButton()							((struct lcd_segmentButton*)lcd_segmentAllocFigure(sizeof(struct lcd_segmentButton)))
#define								lcd_segmentAllocKeyPad()							((struct lcd_segmentKeyPad *)lcd_segmentAllocFigure(sizeof(struct lcd_segmentKeyPad)))
#define								lcd_segmentAllocTenKeyPad()							((struct lcd_segmentTenKeyPad *)lcd_segmentAllocFigure(sizeof(struct lcd_segmentTenKeyPad)))
#define								lcd_segmentAllocSelectionList(n)					((struct lcd_segmentSelectionList *)lcd_segmentAllocFigure(sizeof(struct lcd_segmentSelectionList) + sizeof(struct lcd_segmentButton) * n))
#define								lcd_segmentAllocTextField()							((struct lcd_segmentTextField *)lcd_segmentAllocFigure(sizeof(struct lcd_segmentTextField)))
#define								lcd_segmentAllocIntegerField()						((struct lcd_segmentIntegerField *)lcd_segmentAllocFigure(sizeof(struct lcd_segmentIntegerField)))
#define								lcd_segmentAllocFloatField()						((struct lcd_segmentFloatField *)lcd_segmentAllocFigure(sizeof(struct lcd_segmentFloatField)))
#define								lcd_segmentAllocPullDownField()						((struct lcd_segmentPullDownField *)lcd_segmentAllocFigure(sizeof(struct lcd_segmentPullDownField)))

extern struct lcd_segment *			lcd_segmentNewButton(struct lcd_segmentGroup *g, struct lcd_segmentButton *button);
extern struct lcd_segment *			lcd_segmentNewKeyPad(struct lcd_segmentGroup *g, struct lcd_segmentKeyPad *keyPad);
extern struct lcd_segment *			lcd_segmentNewTenKeyPad(struct lcd_segmentGroup *g, struct lcd_segmentTenKeyPad *keyPad);
extern struct lcd_segment *			lcd_segmentNewSelectionList(struct lcd_segmentGroup *g, struct lcd_segmentSelectionList *list);
extern struct lcd_segment *			lcd_segmentNewTextField(struct lcd_segmentGroup *g, struct lcd_segmentTextField *textField);
extern struct lcd_segment *			lcd_segmentNewIntegerField(struct lcd_segmentGroup *g, struct lcd_segmentIntegerField *field);
extern struct lcd_segment *			lcd_segmentNewFloatField(struct lcd_segmentGroup *g, struct lcd_segmentFloatField *field);
extern struct lcd_segment *			lcd_segmentNewPullDownField(struct lcd_segmentGroup *g, struct lcd_segmentPullDownField *field);

extern int							lcd_segmentSetIntegerFieldValue(struct lcd_segmentIntegerField *g, int i);
extern int							lcd_segmentSetFloatFieldValue(struct lcd_segmentFloatField *g, float f);
extern void							lcd_segmentSetPullDownFieldSelect(struct lcd_segmentPullDownField *g, int select);

#define								lcd_segmentGetIntegerFieldValue(g)					((g)->value)
#define								lcd_segmentGetFloatFieldValue(g)					((g)->value)
#define								lcd_segmentGetPullDownFieldSelect(g)				((g)->select)

#endif /* S95160_S3_H_ */
