//
//  main.c	WHM0.8  Multi-channel watt hour meter (GUI, USB-Bluetooth, USB-Memory)
//  Copyright(C)2013-2014 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2013.05.22 v0.0  First cording
//    2013.11.24 v0.8  Fixed
//    2013.12.27 v0.8a Supported the update bluetooth library 0.1a
//    2014.01.10 v0.8b Supported the update s95160 LCD library 0.1b
//    2014.01.12 v0.8b Fixed
//    2014.01.18 v0.8c An improvement of display performance
//    2014.01.31 v0.8d Modification of save unit(day->hour)
//    2014.02.06 v0.8d The USB memory was supported
//    2014.09.25 v0.8e Improvement of frequency detection precision
//
//
#include <stdio.h>						// libc
#include <stdint.h>						// libc
#include <stddef.h>						// libc
#include <string.h>						// libc
#include <math.h>						// libm
#include <time.h>						// system call
#include <sys/time.h>					// system call

#include "stm32f4_discovery.h"			// public
#include "stm32f4xx_flash.h"			// public
#include "stm32f4xx.h"					// public

#include "usbh_core.h"					// USB host support
#include "usbh_msc_core.h"				// Mass storage class support
#include "usbh_hid_core.h"				// Human interface device class support
#include "usbh_hid_keybd.h"				// Human interface device class support
#include "usbh_hid_mouse.h"				// Human interface device class support
#include "usbh_wcc_bluetooth.h"			// Wireless controller class support
#include "ff.h"							// FAT file system support

#include "bt_process.h"					// Bluetooth control task
#include "bt_rfcomm_server.h"			// Bluetooth RFCOMM API
#include "bt_att_db.h"					// Bluetooth ATT database access API
#include "bt_att_server.h"				// Bluetooth ATT server access API
#include "bt_l2cap_cb_link.h"			// Bluetooth link information

#include "config.h"						// My environment
#include "sensor_data.h"				// Private
#include "analog_capture.h"				// Private
#include "disp.h"						// Private
#include "rtc.h"						// Private
#include "systick.h"					// Private
#include "eprom.h"						// Private
//#define DEBUG

//
//	Define
//
// Inline function
#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define ABS(a)		(((a) >  0 ) ? (a) : -(a))

#ifdef DEBUG
#define DEBUGP(...)		printf(__VA_ARGS__)
#else
#define DEBUGP(...)
#endif


//
//	Data structure
//

//
//	Global area
//
USB_OTG_CORE_HANDLE		USB_OTG_Core;
USBH_HOST				USB_Host;
USBH_Class_cb_TypeDef	USB_Class_Callbacks;
char					*usbDeviceClassName = "";
int						usbDeviceReady;
int						usbMemoryLogInterval = USB_MEMORY_LOG_INTERVAL;
int						usbMemoryStatus;	// -1:error 0:normal 1:remain second of write
int						usbMemorySaveRequest;

struct sensorData		sensorData[CH_COUNT];


//
//	Generate the minus wave
//
#ifdef DEL								// ADD 2014.09.25
int createMinusSignal(short buf[][CH_COUNT], int num, int ch, int period){
	int	h;
	int	i;

	// Getting a wave period
#define WATER_LINE	30
	if (period == 0) {
		int	s1;
		int	s2;
		int	v;

		s1 = -1;
		s2 = -1;
		v  = -1;
		for(i = 0; i < num; i++) {
			if (buf[i][ch] > WATER_LINE) {
				if (v == 0) {			// Rise up point
					if (s1 == -1) {
						s1 = i;			// 1st
					} else  {
						s2 = i;			// 2nd
						break;
					}
				}
				v = 1;
			} else {
				v = 0;
			}
		}
		period = s2 - s1;
		if (period <= 0) {
			period = num;
		}
	}

	// Mirror image
	if ((period >= 2) && (period < num)) {
		h = period / 2;						// half cycle
		for(i = 0; i < num; i++) {
			if (buf[i][ch] < 1) {			//	Low level threshold
				int	p;

				p = i + h;
				if (p >= period) {
					p -= period;
				}
				buf[i][ch] = -buf[p][ch];
			}
		}
	}

	return(period);
}
#else 									// ADD START 2014.09.25
int getPeriod(short buf[][CH_COUNT], int num, int ch){
	int		period;
	short	th;

	// Threshold level
	{									// Average
		int		i;
		long	sum;

		sum = 0;
		for(i = 0; i < num; i++) {
			sum += buf[i][ch];
		}
		th = sum / num;
	}

	// Rise up point #1 to #2
	{
		int	i;
		int	s1;
		int	s2;
		int	v;

		s1 = -1;
		s2 = -1;
		v  = -1;
		for(i = 0; i < num; i++) {
			if (buf[i][ch] > th) {
				if (v == 0) {			// Rise up point
					if (s1 == -1) {
						s1 = i;			// 1st
					} else  {
						s2 = i;			// 2nd
						break;
					}
				}
				v = 1;
			} else {
				v = 0;
			}
		}
		period = s2 - s1;
		if (period <= 0) {				// Fail safe
			period = num;
		}
	}

	return(period);
}

int createMinusSignal(short buf[][CH_COUNT], int num, int ch, int period){
	int	h;
	int	i;

	// Mirror image
	if ((period >= 2) && (period <= num)) {
		h = period / 2;					// half cycle
		for(i = 0; i < num; i++) {
			if (buf[i][ch] <= 0) {		//	Low level threshold
				int	p;

				p = i + h;
				if (p >= period) {
					p -= period;
				}
				buf[i][ch] = -buf[p][ch];
			}
		}
	}

	return(period);
}
#endif									// ADD END 2014.09.25

//
//	Calculate RMS voltage, current, effective power
//
void adcToValue(
		short				adc_val[][CH_COUNT],	// IN: Origin data
		int					num,					// IN: Data count
		struct sensorData	sensorData[])			// IN,OUT: Result(voltage/current/power/frequency)
{
	int	ch;

	for(ch = 0; ch < CH_COUNT; ch++) {
		switch(sensorData[ch].infType){
		case ct:
		{
			short	v;
			short	a;
			short	w;
			short	f;

			// Voltage sensing mode
//			if (sensorData[ch].inf.current.setup.v == 0) {
			if (sensorData[ch].inf.current.setup.vSenseChannel < CH_COUNT) {
				int		vc;
				long	sum_a;
				long	sum_w;
				int		i;

				// Initial
				vc = sensorData[ch].inf.current.setup.vSenseChannel;

				// Time slice
				sum_a = 0;
				sum_w = 0;
				for(i = 0; i < num; i++) {
					short	v;
					short	a;

					// Voltage
					v = adc_val[i][vc];

					// Current
					a = adc_val[i][ch];
					sum_a += ((long)a * (long)a);

					// Power
					sum_w += ((long)v * (long)a);
				}

				// SUMMARY-AVERAGE
				v = sensorData[vc].inf.voltage.v;
				a = (sqrt((double)sum_a / (double)num) * sensorData[ch].inf.current.delta + sensorData[ch].inf.current.offset) * 1000.0;	// [mA] RMS
				w = ((double)sum_w / (double)num * sensorData[vc].inf.voltage.delta * sensorData[ch].inf.current.delta);					// [W] Average
				f = sensorData[vc].inf.voltage.f;
				if (sensorData[ch].inf.current.setup.vSenseMulti) {
					v *= sensorData[ch].inf.current.setup.vSenseMulti;
					w *= sensorData[ch].inf.current.setup.vSenseMulti;
				} else {
					v /= 2;
					w /= 2;
				}

			// Fixed voltage mode
			} else {
				long	sum_a;
				int		i;

				// RMS
				sum_a = 0;
				for(i = 0; i < num; i++) {
					short	a;

					a = adc_val[i][ch];
					sum_a += ((long)a * (long)a);
				}

				v = sensorData[ch].inf.current.setup.v;
				a = (sqrt((double)sum_a / (double)num) * sensorData[ch].inf.current.delta + sensorData[ch].inf.current.offset) * 1000.0;	// [mA]
				w = (long)v * (long)a / 1000L;				// [W]
				f = sensorData[ch].inf.current.setup.f;

			}

			// Store
			sensorData[ch].inf.current.v = v;
			sensorData[ch].inf.current.a = a;
			sensorData[ch].inf.current.w = ABS(w);
			sensorData[ch].inf.current.f = f;

			break;
		}
		case vsense:
		{
			short	v;
			long	sum_v;
			int		i;

			// Time slice
			sum_v  = 0;
			for(i = 0; i < num; i++) {
				short	v;

				// Voltage
				v = adc_val[i][ch];
				sum_v += ((long)v * (long)v);
			}

			// RMS voltage
			v = (sqrt((double)sum_v / (double)num) * sensorData[ch].inf.voltage.delta + sensorData[ch].inf.voltage.offset);

#ifdef FREQ_AVE
			// Frequency filter (Simple moving average)
			{
				static short	pb[FREQ_AVE];	// FIFO buffer
				static int		c = 0;			// Next buffer point
				int				s;
				int				i;

				pb[c++] = sensorData[ch].inf.voltage.f;
				if (c > FREQ_AVE) {
					c = 0;
				}
				s = 0;
				for(i = 0; i < FREQ_AVE; i++) {
					s += pb[i];
				}
				sensorData[ch].inf.voltage.f = s / FREQ_AVE;
			}
#endif

			// Store
			sensorData[ch].inf.voltage.v = v;

			break;
		}
		default:
			break;
		}
	}
	return;
}

// Prepare values
void adcToValuePrepare(struct sensorData sensorData[])
{
	int	ch;

	for(ch = 0; ch < CH_COUNT; ch++) {
		struct sensorData	*p;

		p = &sensorData[ch];
		if (p->infType == ct) {
			// Linear equation convert parameter (y=ax+b: a=delta b=offset)
			p->inf.current.delta  = (double)(ADC_VREF / ADC_RESOLUTION)
								  * (double)p->inf.current.setup.n
								  / (double)p->inf.current.setup.rl
								  * (double)p->inf.current.setup.k;
			p->inf.current.offset = p->inf.current.setup.offset;
			// Fixed values
			p->inf.current.f      = p->inf.current.setup.f;
			p->inf.current.v      = p->inf.current.setup.v;

		} else if (p->infType == vsense) {
			// linear equation convert parameter (y=ax+b: a=delta b=offset)
			p->inf.voltage.delta  = (double)(ADC_VREF / ADC_RESOLUTION)
								  * (double)(p->inf.voltage.setup.r1 + p->inf.voltage.setup.r2)
								  / (double)(p->inf.voltage.setup.r1);
			p->inf.voltage.offset = p->inf.voltage.setup.offset;
		}
	}
	return;
}

//
//	Permanent storage for setting and results
//
#define EPROM_ENTRY_HH_SETTING_FLAG			0x80
#define EPROM_ENTRY_TYPE_DAY_RESULT			1
#define EPROM_ENTRY_TYPE_CURRENT_SENSOR		11
#define EPROM_ENTRY_TYPE_VOLTAGE_SENSOR		12
#define EPROM_ENTRY_TYPE_UNUSED_SENSOR		13
struct eprom {
	unsigned char	yy;			// year-1900
	unsigned char	mm;			// 0-11
	unsigned char	dd;			// 1-31
	unsigned char	hh;			// 0-23

	union __attribute__ ((packed)) {
		struct __attribute__ ((packed)) {
			long			ws[CH_COUNT];	//	[Watt second]
		}	dayResult;

		struct __attribute__ ((packed)) {
			unsigned char				type;
			unsigned char				channel;
			union __attribute__ ((packed)) {
				struct currentSensorSetup	currentSensor;
				struct voltageSensorSetup	voltageSensor;
				char 						unusedSensor;
			}	data;
		}	setting;
	}	entry;
} ;

// Get the EPROM data
struct eprom *getEnergyData(long *ws, int yyy, int mm, int dd, int hh, int opt) {
	struct eprom	*e;
	struct eprom	*r;
	unsigned long	w;
	int				i;

	if (ws) {
		for(i = 0; i < CH_COUNT; i++) {
			*(ws+i) = 0;
		}
	}
	w = 0;
	r = NULL;
	while((e = eprom_getRoundRobin(&w, sizeof(struct eprom))) != NULL) {
		if (!(e->hh & EPROM_ENTRY_HH_SETTING_FLAG)) {
			if ((yyy < 0) || (yyy == e->yy)) {
				if ((mm < 0) || (mm == e->mm)) {
					if ((dd < 0) || (dd == e->dd)) {
						if ((hh < 0) || (hh == e->hh)) {
							if (ws) {
								for(i = 0; i < CH_COUNT; i++) {
									*(ws+i) += e->entry.dayResult.ws[i];
								}
							}
							r = e;
							if (opt) {
								break;
							}
						}
					}
				}
			}
		}
	}
	return(r);
}

//
//	Resuming a data from EPROM
//
void resumeData(struct sensorData sensorData[]){
	int				ch;
	time_t			t;
	struct tm		tm;
	struct timeval	tv;
	unsigned long	w;
	struct eprom	*e;

	// Roll forward by eprom
	printf("EPROM start\n");
	t = 0;
	localtime_r(&t, &tm);
	w = 0;
	while((e = eprom_getRoundRobin(&w, sizeof(struct eprom))) != NULL) {

		printf("%d.%d.%d %d:00:00 -> ", e->yy+1900, e->mm+1, e->dd, e->hh);
		if (!(e->hh & EPROM_ENTRY_HH_SETTING_FLAG)) {
			printf("Result\n");
			if ((tm.tm_year != e->yy) || (tm.tm_mon != e->mm)) {
				for(ch = 0; ch < CH_COUNT; ch++) {
					if (sensorData[ch].infType == ct) {
						sensorData[ch].inf.current.ws_mons[tm.tm_mon] = sensorData[ch].inf.current.ws_mon;
						sensorData[ch].inf.current.ws_mon = 0;
					}
				}
			}
			if (tm.tm_mday != e->dd) {
				for(ch = 0; ch < CH_COUNT; ch++) {
					if (sensorData[ch].infType == ct) {
						sensorData[ch].inf.current.ws_days[tm.tm_mday] = sensorData[ch].inf.current.ws_day;
						sensorData[ch].inf.current.ws_day = 0;
					}
				}
			}
			for(ch = 0; ch < CH_COUNT; ch++) {
				if (sensorData[ch].infType == ct) {
					long	ws;

					ws = e->entry.dayResult.ws[ch];
					sensorData[ch].inf.current.ws_hurs[e->hh]  = ws;
					sensorData[ch].inf.current.ws_hur          = ws;
					sensorData[ch].inf.current.ws_day         += ws;
					sensorData[ch].inf.current.ws_mon         += ws;
				}
			}
			tm.tm_year = e->yy;
			tm.tm_mon  = e->mm;
			tm.tm_mday = e->dd;
			tm.tm_hour = e->hh;

		} else {
			printf("Setup CH#%d=%d\n", e->entry.setting.channel, e->entry.setting.type);
			ch = e->entry.setting.channel;
			switch(e->entry.setting.type) {
			case EPROM_ENTRY_TYPE_CURRENT_SENSOR:
				sensorData[ch].infType = ct;
				memcpy(&(sensorData[ch].inf.current.setup), &(e->entry.setting.data.currentSensor), sizeof(struct currentSensorSetup));
				break;

			case EPROM_ENTRY_TYPE_VOLTAGE_SENSOR:
				sensorData[ch].infType = vsense;
				memcpy(&(sensorData[ch].inf.voltage.setup), &(e->entry.setting.data.voltageSensor), sizeof(struct voltageSensorSetup));
				break;

			case EPROM_ENTRY_TYPE_UNUSED_SENSOR:
				sensorData[ch].infType = notConnect;
				break;

			default:
				break;
			}
		}
	}

	// Updating a RTC
	t = mktime(&tm) + 3600;	// Next 1hour
	if (t > time(NULL)) {
		tv.tv_sec  = t;
		tv.tv_usec = 0;
		settimeofday(&tv, NULL);
	}
	{
		struct tm	new;

		localtime_r(&t, &new);
		if ((tm.tm_year != new.tm_year) || (tm.tm_mon != new.tm_mon)) {
			for(ch = 0; ch < CH_COUNT; ch++) {
				if (sensorData[ch].infType == ct) {
					sensorData[ch].inf.current.ws_mon = 0;
				}
			}
		}
		if (tm.tm_mday != new.tm_mday) {
			for(ch = 0; ch < CH_COUNT; ch++) {
				if (sensorData[ch].infType == ct) {
					sensorData[ch].inf.current.ws_day = 0;
				}
			}
		}
		printf("New date %d.%d.%d %d:00:00\n", new.tm_year+1900, new.tm_mon+1, new.tm_mday, new.tm_hour);
	}

	// Setting values
	adcToValuePrepare(sensorData);

	// EPROM Append point
	eprom_setRoundRobinNewEntry(w);

	printf("EPROM end\n");
	return;
}

//
//	Save to EPROM
//
void saveSetup(struct sensorData *d, int ch){
	struct eprom	e;
	time_t			t;
	struct tm		tm;

	// Build an EPROM format
	time(&t);
	localtime_r(&t, &tm);
	memset(&e, 0xff, sizeof(e));
	e.yy = tm.tm_year;
	e.mm = tm.tm_mon;
	e.dd = tm.tm_mday;
	e.hh = tm.tm_hour | EPROM_ENTRY_HH_SETTING_FLAG;
	e.entry.setting.channel = ch;
	switch(d->infType) {
	case ct:
		e.entry.setting.type = EPROM_ENTRY_TYPE_CURRENT_SENSOR;
		memcpy(&e.entry.setting.data.currentSensor, &(d->inf.current.setup), sizeof(e.entry.setting.data.currentSensor));
		break;
	case vsense:
		e.entry.setting.type = EPROM_ENTRY_TYPE_VOLTAGE_SENSOR;
		memcpy(&e.entry.setting.data.voltageSensor, &(d->inf.voltage.setup), sizeof(e.entry.setting.data.voltageSensor));
		break;
	default:
		e.entry.setting.type = EPROM_ENTRY_TYPE_UNUSED_SENSOR;
		break;
	}

	// Update
	eprom_appendRoundRobin(&e, sizeof(e));
	return;
}

//
//	USB memory stick I/O
//
int usbMemoryProcess(int flag){
	static int		state;
	static FATFS	fs;
	int				rc;

#ifdef DEL
	{
		static int	stateBefore;

		if (stateBefore != state) {
			stateBefore = state;
			DEBUGP("new state=%d\n", state);
		}
	}
#endif

	rc = 0;
	if (flag) {
		state = 0;
	}
	switch(state){
	case 0:
		if (*usbDeviceClassName == 'S') {
			state = 1;
			printf("USB: Found mass storage\n");
		}
		break;

	case 1:
		if (f_mount(0, &fs) == FR_OK) {
			state = 10;
		} else {
			state = 99;
			printf("USB: FAT Mount error\n");
		}
		break;

	case 10:		// EPROM Restore
	{
		unsigned long	addr;
		void	Reset_Handler(void);

		printf("USB: Check, save file\n");
		addr = 0;
		if (eprom_getRoundRobin(&addr, sizeof(struct eprom)) == NULL) {
			FIL	fd;

			if (f_open(&fd, USB_MEMORY_SAVE_FILENAME, FA_READ) == FR_OK) {
				struct eprom	data;
				UINT	l;
				int		i;

				i = 0;
				printf("EPROM Restore\n");
				while(f_read(&fd, &data, sizeof(data), &l) == FR_OK) {
					if (l < sizeof(data)) {
						break;
					}
					i++;
					printf("Write  %d\n", i);
					if (eprom_appendRoundRobin(&data, sizeof(data)) != 0) {
						break;
					}
				}
				f_close(&fd);
				printf("Please reset.\n");
				Reset_Handler();
			}
		}
		state = 20;
		break;
	}

	case 20:		// Logging
	{
		static time_t	ssBefore;
		time_t			t;

		state = 30;
		time(&t);
		while(ssBefore != t) {
			char	fname[32];
			FRESULT	rc;
			FIL		fd;
			UINT	l;
			char 	buf[400];
			int		i;
			struct sensorData	*d;
			struct tm			tm;
			int		sec;
			int		rem;

			ssBefore = t;
			localtime_r(&t, &tm);
			sec = tm.tm_hour * 3600 + tm.tm_min * 60 + tm.tm_sec;
			rem = usbMemoryLogInterval - (sec % usbMemoryLogInterval);
			if (rem < usbMemoryLogInterval) {
				if (usbMemoryStatus >= 0) {
					usbMemoryStatus = rem;
				}
				break;
			}
			usbMemoryStatus = rem;

//			printf("f_open\n");
			sprintf(fname, USB_MEMORY_LOG_FILENAME, tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday);
			if ((rc = f_open(&fd, fname,  FA_OPEN_ALWAYS | FA_WRITE)) != FR_OK) {
				printf("f_open err rc=%d\n", rc);
				state = 99;
				break;
			}
			if (fd.fsize == 0) {
				strcpy(buf, "\"Date Time\",");
				for(d = sensorData, i = 0; i < CH_COUNT; i++, d++) {
					char a[80];

					if (d->infType == ct) {
						sprintf(a, "\"%s Energy[kWh]\",\"%s Power[W]\",\"%s Current[A]\",", d->inf.current.setup.name, d->inf.current.setup.name, d->inf.current.setup.name);
					} else if (d->infType == vsense) {
						sprintf(a, "\"CH%d Voltage[V]\",\"CH%d Frequency[Hz]\",", i+1, i+1);
					}
					strcat(buf, a);
				}
				strcat(buf, "\n");

				if ((rc = f_write(&fd, buf, strlen(buf), &l)) != FR_OK) {
					printf("head f_write err rc=%d len=%d\n", rc, l);
					state = 99;
					break;
				}
			} else {
//				printf("f_lseek\n");
				if ((rc = f_lseek(&fd, fd.fsize)) != FR_OK) {
					printf("f_seek err rc=%d fsize=%ld\n", rc, fd.fsize);
					state = 99;
					break;
				}
			}

			sprintf(buf, "\"%04d-%02d-%02d %02d:%02d:%02d\",", tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
			for(d = sensorData, i = 0; i < CH_COUNT; i++, d++) {
				char a[32];

				if (d->infType == ct) {
					sprintf(a, "%.2f,%d,%.3f,", d->inf.current.ws_mon/3600000.0, d->inf.current.w, d->inf.current.a/1000.0);
				} else if (d->infType == vsense) {
					sprintf(a, "%d,%d,", d->inf.voltage.v, d->inf.voltage.f);
				}
				strcat(buf, a);
			}
			strcat(buf, "\n");
//			printf("f_write\n");
			if ((rc = f_write(&fd, buf, strlen(buf), &l)) != FR_OK) {
				printf("data f_write err rc=%d\n", rc);
				state = 99;
			}
//			printf("f_close\n");
			if ((rc = f_close(&fd)) != FR_OK) {
				printf("f_close err rc=%d\n", rc);
				state = 99;
			}
//			printf("return\n");

			break;
		}
		break;
	}

	case 30:		// EPROM Backup
	{
		if (usbMemorySaveRequest) {
			FIL				fd;
			UINT			l;
			unsigned char	*e;
			unsigned long	addr;

			usbMemorySaveRequest = 0;
			if (f_open(&fd, USB_MEMORY_SAVE_FILENAME, FA_WRITE|FA_CREATE_ALWAYS) == FR_OK) {
				addr = 0;
				while((e = eprom_getRoundRobin(&addr, sizeof(struct eprom))) != NULL) {
					if (f_write(&fd, e, sizeof(struct eprom), &l) != FR_OK) {
						break;
					}
				}
				f_close(&fd);
			}
		}
		state = 20;
		break;
	}

	default:
		usbMemoryStatus = -1;
		state = 0;
		rc = 1;
		break;
	}

	return(rc);
}


//=====================================================================
//
//	USB host callback routines
//
//=====================================================================

void USBH_USR_Init(void){
	DEBUGP("USBH_USR_Init\n");
}

void USBH_USR_DeviceAttached(void){
	DEBUGP("USBH_USR_DeviceAttached\n");
}

void USBH_USR_UnrecoveredError(void){
	DEBUGP("USBH_USR_UnrecoveredError\n");
}

void USBH_USR_DeviceDisconnected(void){
	DEBUGP("USBH_USR_DeviceDisconnected\n");
	STM_EVAL_LEDOff(LED6);		// Blue
	usbDeviceClassName = "";
	usbDeviceReady     = 0;
}

void USBH_USR_ResetDevice(void){
	DEBUGP("USBH_USR_ResetDevice\n");
}

void USBH_USR_DeviceSpeedDetected(uint8_t DeviceSpeed){
	DEBUGP("USBH_USR_DeviceSpeedDetected: ");
	switch(DeviceSpeed){
	case HPRT0_PRTSPD_HIGH_SPEED:
		DEBUGP("High speed device\n");
		break;
	case HPRT0_PRTSPD_FULL_SPEED:
		DEBUGP("Full speed device\n");
		break;
	case HPRT0_PRTSPD_LOW_SPEED:
		DEBUGP("Low speed device\n");
		break;
	default:
		DEBUGP("fault\n");
		break;
	}
}

void USBH_USR_Device_DescAvailable(void *DeviceDesc){
	DEBUGP("USBH_USR_Device_DescAvailable: ");
	DEBUGP("VID=0x%04x " , ((USBH_DevDesc_TypeDef *)DeviceDesc)->idVendor);
	DEBUGP("PID=0x%04x\n" , ((USBH_DevDesc_TypeDef *)DeviceDesc)->idProduct);
}

void USBH_USR_DeviceAddressAssigned(void){
	DEBUGP("USBH_USR_DeviceAddressAssigned\n");
}

void USBH_USR_Configuration_DescAvailable(
	USBH_CfgDesc_TypeDef		*cfgDesc,
    USBH_InterfaceDesc_TypeDef	*itfDesc,
    USBH_EpDesc_TypeDef			*epDesc
){
	if (itfDesc->bInterfaceClass == USB_MSC_CLASS) {
		usbMemoryProcess(1);
		memcpy(&USB_Class_Callbacks, &USBH_MSC_cb, sizeof(USB_Class_Callbacks));
		usbDeviceClassName = "Storage";
	} else if (itfDesc->bInterfaceClass == USB_HID_CLASS) {
		memcpy(&USB_Class_Callbacks, &HID_cb, sizeof(USB_Class_Callbacks));
		usbDeviceClassName = "HID";
	} else if (itfDesc->bInterfaceClass == USB_WCC_CLASS) {
		memcpy(&USB_Class_Callbacks, &USBH_WCC_Bluetooth_cb, sizeof(USB_Class_Callbacks));
		usbDeviceClassName = "Wireless";
	} else {
		DEBUGP("Unsupport class\n");
		usbDeviceClassName = "Unknown";
	}
	usbMemoryStatus = 0;
	DEBUGP("USBH_USR_Configuration_DescAvailable: class=%s\n", usbDeviceClassName);
}
void  USR_KEYBRD_Init(void){
	printf("USR_KEYBRD_Init\n");
	return;								// HID class dummy
}
void  USR_KEYBRD_ProcessData(uint8_t pbuf){
	printf("USR_KEYBRD_ProcessData: 0x%02x\n", pbuf);
	return;								// HID class dummy
}
void  USR_MOUSE_Init (void){
	printf("USR_MOUSE_Init\n");
	return;								// HID class dummy
}
void  USR_MOUSE_ProcessData(HID_MOUSE_Data_TypeDef *data){
	printf("USR_MOUSE_ProcessData: (%d,%d,%d) %d\n", data->x, data->y, data->z, data->button);
	return;								// HID class dummy
}

void USBH_USR_Manufacturer_String(void *ManufacturerString){
	DEBUGP("USBH_USR_Manufacturer_String: %s\n", (char *)ManufacturerString);
}

void USBH_USR_Product_String(void *ProductString){
	DEBUGP("USBH_USR_Product_String: %s\n", (char *)ProductString);
}

void USBH_USR_SerialNum_String(void *SerialNumString){
	DEBUGP("USBH_USR_SerialNum_String: %s\n", (char *)SerialNumString);
}

void USBH_USR_EnumerationDone(void){
	DEBUGP("USBH_USR_EnumerationDone\n");
}

void USBH_USR_DeviceNotSupported(void){
	printf("USBH_USR_DeviceNotSupported\n");
}

USBH_USR_Status USBH_USR_UserInput(void){
	DEBUGP("USBH_USR_UserInput\n");
	STM_EVAL_LEDOn(LED6);		// Blue
	usbDeviceReady = 1;
	return(USBH_USR_RESP_OK);
}

int USBH_USR_UserApplication(void){
//	DEBUGP("USBH_USR_UserApplication\n");
	usbMemoryProcess(0);
	return(0);
}

void USBH_USR_OverCurrentDetected (void){
	printf("USBH_USR_OverCurrentDetected\n");
}

void USBH_USR_DeInit(void){
	DEBUGP("USBH_USR_DeInit\n");
}

void USBH_USR_Dummy(void){
}


//=====================================================================
//
//	USB memory
//
//=====================================================================
// ff.h: FatFs custom code
DWORD get_fattime(void) {
	time_t		t;
	struct tm	tm;

	t = time(NULL);
	localtime_r(&t, &tm);
	if (tm.tm_year < 80) {
		tm.tm_year = 80;
	}
	return(((tm.tm_year+1900-1980) << 25) |
	       ((tm.tm_mon+1) << 21) |
	       (tm.tm_mday << 16) |
	       (tm.tm_hour << 11) |
	       (tm.tm_min  << 5) |
	       (tm.tm_sec / 2)
			);
}


//=====================================================================
//
//	USB-Bluetooth callback routines
//
//=====================================================================

//	RFCOMM User Own Cording
void bt_rfcommServerDataRecived(unsigned char *inBuff, size_t inLen){
	;									// User code
	return;
}
// ATT DB manager Callback	(Ref. bt_att_db.c)
void bt_attDbNotification(unsigned short handle, unsigned char *value, size_t valueLen){

	// Essential action
	bt_att_serverNotificationQueueAdd(handle);

	// User action
	switch(handle) {
	case 0x1002:		// Change a date time
	{
		struct datetime {
			short			yyyy;
			unsigned char	mm;
			unsigned char	dd;
			unsigned char	hh;
			unsigned char	mi;
			unsigned char	ss;
		}	*datetime;
		struct timeval	tv;
		struct tm		tm;

		datetime = (struct datetime *)value;
		tm.tm_year = datetime->yyyy - 1900;
		tm.tm_mon  = datetime->mm - 1;
		tm.tm_mday = datetime->dd;
		tm.tm_hour = datetime->hh;
		tm.tm_min  = datetime->mi;
		tm.tm_sec  = datetime->ss;
		tv.tv_sec  = mktime(&tm);
		tv.tv_usec = 0;
		settimeofday(&tv, NULL);
		break;
	}
	case 0x1102:		// Request for history data
	{
		struct reqDate {
			short		yyyy;
			signed char	mm;
			signed char	dd;
			signed char	hh;
		}	*reqDate;
		unsigned short	ansData[CH_COUNT];
		long			ws[CH_COUNT];

		reqDate = (struct reqDate *)&value;
		if (getEnergyData(ws, reqDate->yyyy-1900, reqDate->mm, reqDate->dd, reqDate->hh, 0) != NULL) {
			int	ch;

			for(ch = 0; ch < CH_COUNT; ch++) {
				ansData[ch] = ws[ch] / 3600;	// Ws -> Wh
			}
			bt_attDbUpdate(0x1112, (unsigned char *)&ansData, sizeof(ansData));
		}
		break;
	}
	case 0x1122:		// Request for setting data
	{
		struct reqData {
			unsigned char	cmd;
			unsigned char	ch;
			unsigned char	setup[18];
		}	*reqData;
		struct {
			unsigned char	ch;
			unsigned char	type;
			unsigned char	setup[18];
		}	ansData;
		int	ch;

		reqData = (struct reqData *)&value;
		ch = reqData->ch;
		switch(reqData->cmd) {
		case 1:			// Read setup data
			ansData.ch = ch;
			ansData.type = sensorData[ch].infType;
			if (sensorData[ch].infType == ct) {
				memcpy(ansData.setup, &(sensorData[ch].inf.current.setup), sizeof(ansData.setup));
			} else if (sensorData[ch].infType == vsense) {
				memcpy(ansData.setup, &(sensorData[ch].inf.voltage.setup), sizeof(ansData.setup));
			} else {
				memset(ansData.setup, 0, sizeof(ansData.setup));
			}
			bt_attDbUpdate(0x1132, (unsigned char *)&ansData, sizeof(ansData));
			break;
		case 2:			// Read channel name data
			ansData.ch = ch;
			ansData.type = sensorData[ch].infType;
			if (sensorData[ch].infType == ct) {
				memcpy(ansData.setup, &(sensorData[ch].inf.current.setup.name), sizeof(ansData.setup));
			} else {
				strcpy((char *)ansData.setup, "N/A");
			}
			bt_attDbUpdate(0x1132, (unsigned char *)&ansData, sizeof(ansData));
			break;
		case 11:		// Write current setup data
			if ((ch >= 0) && (ch < CH_COUNT)) {
				sensorData[ch].infType = ct;
				memcpy(&(sensorData[ch].inf.current.setup), reqData->setup, sizeof(reqData->setup));
				adcToValuePrepare(sensorData);
				saveSetup(&(sensorData[ch]), ch);
			}
			break;
		case 12:		// Write voltage setup data
			if ((ch >= 0) && (ch < CH_COUNT)) {
				sensorData[ch].infType = vsense;
				memcpy(&(sensorData[ch].inf.voltage.setup), reqData->setup, sizeof(reqData->setup));
				adcToValuePrepare(sensorData);
				saveSetup(&(sensorData[ch]), ch);
			}
			break;
		case 13:		// Write unuse set
			if ((ch >= 0) && (ch < CH_COUNT)) {
				sensorData[ch].infType = notConnect;
				adcToValuePrepare(sensorData);
				saveSetup(&(sensorData[ch]), ch);
			}
			break;
		case 21:		// Write channel name
			if ((ch >= 0) && (ch < CH_COUNT)) {
				if (sensorData[ch].infType == ct) {
					memcpy(sensorData[ch].inf.current.setup.name, reqData->setup, sizeof(sensorData[ch].inf.current.setup.name));
					saveSetup(&(sensorData[ch]), ch);
				}
			}
			break;
		default:
			break;
		}
		break;
	}
	default:
		break;
	}
	return;
}


//=====================================================================
//
//		M A I N
//
//=====================================================================
int main(void){
	USBH_Usr_cb_TypeDef	cb ={
	  USBH_USR_Init,
	  USBH_USR_DeInit,
	  USBH_USR_DeviceAttached,
	  USBH_USR_ResetDevice,
	  USBH_USR_DeviceDisconnected,
	  USBH_USR_OverCurrentDetected,
	  USBH_USR_DeviceSpeedDetected,
	  USBH_USR_Device_DescAvailable,
	  USBH_USR_DeviceAddressAssigned,
	  USBH_USR_Configuration_DescAvailable,
	  USBH_USR_Manufacturer_String,
	  USBH_USR_Product_String,
	  USBH_USR_SerialNum_String,
	  USBH_USR_EnumerationDone,
	  USBH_USR_UserInput,
	  USBH_USR_UserApplication,
	  USBH_USR_DeviceNotSupported,
	  USBH_USR_UnrecoveredError
	};
	uint16_t		adc_buf[SAMPLE_COUNT][CH_COUNT];	// ADC auto captured data
	short			adc_val[SAMPLE_COUNT][CH_COUNT];
	long			s_day;				// [s]
	long			s_mon;				// [s]
	struct tm		tm;
	time_t			t;
	time_t			t_before;			//
	int				hur_before;			//
	int				day_before;			//
	int				mon_before;			//
	int				year_before;		//

	enum appState_t {
		APP_STATE_IDLE = 0,
		APP_STATE_PREPROCESS,
		APP_STATE_CAPTURE,
		APP_STATE_LPF,
		APP_STATE_MIRROR,
		APP_STATE_CALCULATE,
		APP_STATE_ESTIMATE,
		APP_STATE_BLUETOOTH,
		APP_STATE_POSTPROCESS,
		APP_STATE_DISPLAY,
		APP_STATE_INPUT,
		APP_STATE_ACTION,
	}	appState;
	int				i;

	//
	// Board Init
	//
	STM_EVAL_LEDInit(LED3);				// Orange
	STM_EVAL_LEDInit(LED4);				// Green
	STM_EVAL_LEDInit(LED5);				// Red
	STM_EVAL_LEDInit(LED6);				// Blue
    STM_EVAL_PBInit(BUTTON_USER, BUTTON_MODE_GPIO);

    // Start date and time clock
	rtc_open();

	// Start a tick time
	systick_open();

	// Start a EEPROM
	eprom_open();

	// Start an analog block
	analogCaptureFreeRun(adc_buf, SAMPLE_COUNT, (int)(SAMPLE_INTERVAL*1000000));

	// Start a USB host
#ifdef USE_USB_OTG_FS
	USBH_Init(&USB_OTG_Core, USB_OTG_FS_CORE_ID, &USB_Host, &USB_Class_Callbacks, &cb);
#else
	USBH_Init(&USB_OTG_Core, USB_OTG_HS_CORE_ID, &USB_Host, &USB_Class_Callbacks, &cb);
#endif

	//
	// APP Init
	//
	{
		int		ch;

		printf("WHM0.8e (C)2014 Toyohiko Togashi\n");
		memset(&sensorData, 0, sizeof(sensorData));
		for(ch = 0; ch < CH_COUNT; ch++) {		// Default setup
#ifdef VSENSE_CH_DFLT
			if (ch == VSENSE_CH_DFLT) {
				sensorData[ch].infType = vsense;
				sensorData[ch].inf.voltage.setup.offset = 0.0;
				sensorData[ch].inf.voltage.setup.r1     = LINE_VOLT_DIV_R1;
				sensorData[ch].inf.voltage.setup.r2     = LINE_VOLT_DIV_R2;
			} else {
				sensorData[ch].infType = ct;
				sprintf(sensorData[ch].inf.current.setup.name, "CH%d", ch+1);
				sensorData[ch].inf.current.setup.rl            = CT_RL;
				sensorData[ch].inf.current.setup.n             = CT_N;
				sensorData[ch].inf.current.setup.k             = CT_K;
				sensorData[ch].inf.current.setup.f             = LINE_FREQ_DFLT;
				sensorData[ch].inf.current.setup.v             = LINE_VOLT_DFLT;
				sensorData[ch].inf.current.setup.vSenseChannel = VSENSE_CH_DFLT;
				sensorData[ch].inf.current.setup.vSenseMulti   = 1;
			}
#else
			sensorData[ch].infType = ct;
			sprintf(sensorData[ch].inf.current.setup.name, "CH%d", ch+1);
			sensorData[ch].inf.current.setup.rl            = CT_RL;
			sensorData[ch].inf.current.setup.n             = CT_N;
			sensorData[ch].inf.current.setup.k             = CT_K;
			sensorData[ch].inf.current.setup.f             = LINE_FREQ_DFLT;
			sensorData[ch].inf.current.setup.v             = LINE_VOLT_DFLT;
			sensorData[ch].inf.current.setup.vSenseChannel = CH_COUNT;
			sensorData[ch].inf.current.setup.vSenseMulti   = 1;
#endif
		}

		// Recall data and setup
		resumeData(sensorData);		// Resuming a data, settings and date-time
		time(&t_before);
		localtime_r(&t_before, &tm);
		hur_before    = tm.tm_hour;
		day_before    = tm.tm_mday;
		mon_before    = tm.tm_mon;
		year_before   = tm.tm_year;
		s_day         = 0;
		s_mon         = 0;
		appState      = APP_STATE_IDLE;

		// Display init
		dispOpen((short *)adc_val, sensorData, &tm);

		printf("Pause 3 second\n");
		while(time(NULL) < (t_before + 3)) {
			USBH_Process(&USB_OTG_Core, &USB_Host);		// USB host task
			bt_process();								// Bluetooth task
		}
	}

	//
	//	Grand event loop
	//
	for(i = 0; ; i++) {
		int	rc;
		int	f;

		// Hart beat LED
		if (i > 10000) {
			i = 0;
			STM_EVAL_LEDToggle(LED3);		// Orange
		}

		// USB host task
		USBH_Process(&USB_OTG_Core, &USB_Host);

		// Bluetooth task
		rc = bt_process();
		if (rc < 0) {
			STM_EVAL_LEDToggle(LED5);		// Red
		}
		if (STM_EVAL_PBGetState(BUTTON_USER)) {
			STM_EVAL_LEDOff(LED5);			// Red
		}

		//
		//	APP task
		//
		switch(appState){
		case APP_STATE_IDLE:
			time(&t);
			if (t_before != t) {			// Hook of second timing
				localtime_r(&t, &tm);
				appState = APP_STATE_PREPROCESS;
			} else {
				appState = APP_STATE_DISPLAY;
			}
			break;

		case APP_STATE_PREPROCESS:
			//	Year interval
			f = 0;
			if (year_before != tm.tm_year) {
				f = 1;
			}
			//	Month interval
			if ((mon_before != tm.tm_mon) || f) {
				int	ch;

				for(ch = 0; ch < CH_COUNT; ch++) {
					if (sensorData[ch].infType == ct) {
						sensorData[ch].inf.current.ws_mons[mon_before] = sensorData[ch].inf.current.ws_mon;
						sensorData[ch].inf.current.ws_mon = 0;
					}
				}
				s_mon = 0;
				f = 1;
			}
			//	Day interval
			if ((day_before != tm.tm_mday) || f) {
				int	ch;

				for(ch = 0; ch < CH_COUNT; ch++) {
					if (sensorData[ch].infType == ct) {
						sensorData[ch].inf.current.ws_days[day_before] = sensorData[ch].inf.current.ws_day;
						sensorData[ch].inf.current.ws_day = 0;
					}
				}
				s_day = 0;

				f = 1;
			}
			//	Hour interval
			if ((hur_before != tm.tm_hour) || f) {
				int	ch;

				if (year_before > 100) {	// 2000~ Record to EPROM
					struct eprom	ent;
					int				rc;

					ent.yy = year_before;
					ent.mm = mon_before;
					ent.dd = day_before;
					ent.hh = hur_before;
					for(ch = 0; ch < CH_COUNT; ch++) {
						if (sensorData[ch].infType == ct) {
							ent.entry.dayResult.ws[ch] = sensorData[ch].inf.current.ws_hur;
						} else {
							ent.entry.dayResult.ws[ch] = 0;
						}
					}
					if ((rc = eprom_appendRoundRobin(&ent, sizeof(ent))) != 0) {
						printf("EPROM Write error %d\n", rc);
						printf("STOP\n");
						for(;;);
					}
				}

				for(ch = 0; ch < CH_COUNT; ch++) {
					if (sensorData[ch].infType == ct) {
						sensorData[ch].inf.current.ws_hurs[hur_before] = sensorData[ch].inf.current.ws_hur;
						sensorData[ch].inf.current.ws_hur = 0;
					}
				}
				f = 1;
			}
			//	Sec interval
			appState = APP_STATE_CAPTURE;
			break;

		// Snapshot a wave form
		case APP_STATE_CAPTURE:
			memcpy(adc_val, adc_buf, sizeof(adc_val));
			appState = APP_STATE_LPF;
			break;

		// LPF(Low Pass Filter)
		case APP_STATE_LPF:
#ifdef SPIKE_CANCELER					// ADD START 2014.09.25
		{
			int	i;
			int	j;

			for(i = 0; i < CH_COUNT; i++) {
				for(j = 1; j < (SAMPLE_COUNT-1); j++) {
					short	l;
					short	h;

					if (adc_val[j-1][i] < adc_val[j+1][i]) {
						l = adc_val[j-1][i];
						h = adc_val[j+1][i];
					} else {
						l = adc_val[j+1][i];
						h = adc_val[j-1][i];
					}
					if ((adc_val[j][i] < l) || (adc_val[j][i] > h)) {
						adc_val[j][i] = (h + l) / 2;
					}
				}
			}
		}
#endif // SPIKE_CANCELER				// ADD END 2014.09.25
#ifdef LPF_RATE
			{
				int	i;
				int	j;
				int	k;

				for(i = 0; i < CH_COUNT; i++) {
					for(j = 0; j < (SAMPLE_COUNT-LPF_RATE); j++) {
//						if (adc_val[j][i] > 0) {	// DEL 2014.09.25
//							short	s;				// DEL 2014.09.25
							long	s;				// ADD 2014.09.25

							s = 0;
							for(k = 0; k < LPF_RATE; k++) {
//								s += (adc_val[j+k][i] - 1);	// DEL 2014.09.25
								s += adc_val[j+k][i];		// ADD 2014.09.25
							}
							adc_val[j][i] = s / LPF_RATE;
//						}							// DEL 2014.09.25
					}
				}
			}
#endif	// LPF_RATE
			appState = APP_STATE_MIRROR;
			break;

		// Mirroring of half wave
		case APP_STATE_MIRROR:
#ifdef DEL								// ADD 2014.09.25
			{
				int	ch;
				int	period;

				for(ch = 0; ch < CH_COUNT; ch++) {
					switch(sensorData[ch].infType){
					case ct:
						period = (int)(1.0/SAMPLE_INTERVAL) / sensorData[ch].inf.current.f;
						createMinusSignal(adc_val, SAMPLE_COUNT, ch, period);
						break;
					case vsense:
						period = createMinusSignal(adc_val, SAMPLE_COUNT, ch, 0);
						sensorData[ch].inf.voltage.f = 1.0 / (SAMPLE_INTERVAL * period);
						break;
					default:
						break;
					}
				}
			}
#else									// ADD START 2014.09.25
			{
				int	ch;
				int	f;

				// Get a frequency
#define UNDEFINED_FREQUENCY	60
				f = UNDEFINED_FREQUENCY;								// Default value
				for(ch = 0; ch < CH_COUNT; ch++) {
					int	period;

					if (sensorData[ch].infType == vsense) {
						period = getPeriod(adc_val, SAMPLE_COUNT, ch);
						f      = 1.0 / (SAMPLE_INTERVAL * period);
						break;
					}
				}

				// Setting the frequency value
				for(ch = 0; ch < CH_COUNT; ch++) {
					switch(sensorData[ch].infType) {
					case vsense:
						sensorData[ch].inf.voltage.f = f;
						break;
					case ct:
						if (sensorData[ch].inf.current.setup.f == 0) {	// Voltage sense mode
							sensorData[ch].inf.current.f = f;
						} else {
							sensorData[ch].inf.current.f = sensorData[ch].inf.current.setup.f;
						}
						break;
					default:
						break;
					}
				}

				// Mirror of the wave form
				for(ch = 0; ch < CH_COUNT; ch++) {
					int	period;

					switch(sensorData[ch].infType) {
					case vsense:
						period = (int)(1.0/SAMPLE_INTERVAL) / sensorData[ch].inf.voltage.f;
						createMinusSignal(adc_val, SAMPLE_COUNT, ch, period);
						break;
					case ct:
						period = (int)(1.0/SAMPLE_INTERVAL) / sensorData[ch].inf.current.f;
						createMinusSignal(adc_val, SAMPLE_COUNT, ch, period);
						break;
					default:
						break;
					}
				}
			}
#endif									// ADD END 2014.09.25
			appState = APP_STATE_CALCULATE;
			break;

		// Calculation of electric power
		case APP_STATE_CALCULATE:
			adcToValue(adc_val, SAMPLE_COUNT, sensorData);
			appState = APP_STATE_ESTIMATE;
			break;

		// Estimate
		case APP_STATE_ESTIMATE:
			{
				int	ch;

				for(ch = 0; ch < CH_COUNT; ch++) {
					if (sensorData[ch].infType == ct) {
						sensorData[ch].inf.current.ws_hur += sensorData[ch].inf.current.w;
						sensorData[ch].inf.current.ws_day += sensorData[ch].inf.current.w;
						sensorData[ch].inf.current.ws_mon += sensorData[ch].inf.current.w;
					}
				}
				s_day++;
				s_mon++;
			}
			appState = APP_STATE_BLUETOOTH;
			break;

		// Bluetooth ATT database update and notification
		case APP_STATE_BLUETOOTH:
#ifdef BT_SEND_TIME
			{							// Date time
				struct {
					short			yyyy;
					unsigned char	mm;
					unsigned char	dd;
					unsigned char	hh;
					unsigned char	mi;
					unsigned char	ss;
				}			datetime;

				datetime.yyyy = tm.tm_year + 1900;
				datetime.mm   = tm.tm_mon + 1;
				datetime.dd   = tm.tm_mday;
				datetime.hh   = tm.tm_hour;
				datetime.mi   = tm.tm_min;
				datetime.ss   = tm.tm_sec;
				bt_attDbUpdate(0x1002, (unsigned char *)&datetime, sizeof(datetime));
			}
#endif	// DEL
			{							// Real time data
				struct {
					float			w;
					float			a;
					float			v;
					float			f;
					float			e;
				}			powerdata;
				const short	handle[] = {0x1012,0x1022,0x1032,0x1042,0x1052,0x1062,0x1072};
				int			ch;

				for(ch = 0; ch < CH_COUNT; ch++) {
					if (sensorData[ch].infType == ct) {
						powerdata.w = sensorData[ch].inf.current.w;				// W
						powerdata.a = sensorData[ch].inf.current.a / 1000.0;	// mA -> A
						powerdata.v = sensorData[ch].inf.current.v;				// V
						powerdata.f = sensorData[ch].inf.current.f;				// Hz
						powerdata.e = sensorData[ch].inf.current.ws_mon / (float)(60 * 60);	// Ws -> Wh

						bt_attDbUpdate(handle[ch], (unsigned char *)&powerdata, sizeof(powerdata));
					}
				}
			}
			appState = APP_STATE_POSTPROCESS;
			break;

		case APP_STATE_POSTPROCESS:
			//	Second interval
			//	Hour interval
			if (hur_before != tm.tm_hour) {
				hur_before  = tm.tm_hour;		// Important
			}
			//	Day interval
			if (day_before != tm.tm_mday) {
				day_before  = tm.tm_mday;		// Important
			}
			//	Month interval
			if (mon_before != tm.tm_mon) {
				mon_before  = tm.tm_mon;		// Important
			}
			//	Year interval
			if (year_before != tm.tm_year) {
				year_before  = tm.tm_year;		// Important
			}
			t_before = t;
			appState = APP_STATE_IDLE;
			break;

		case APP_STATE_DISPLAY:
			dispRefresh();
			appState = APP_STATE_INPUT;
			break;

		case APP_STATE_INPUT:
			appState = APP_STATE_ACTION;
			break;

		case APP_STATE_ACTION:
			appState = APP_STATE_IDLE;
			break;

		default:
			appState = APP_STATE_IDLE;
			break;
		}

		//
		//	End of loop
		//
	}
}


