//
//  usbh_wcc_bluetooth.c
//
//  USB Host Protocol Stack - USB Wireless controller class for bluetooth dongle
//  Copyright (C) 2013 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Reference:
//		Bluetooth SIG (www.bluetooth.com)
//			BLUETOOTH SPECIFICATION Version 4.0
//				[Vol 4] Host Controller Interface [Transport Layer]
//				Part B: USB Transport Layer
//		STMicroelectronics
//			STM32 USB HOST Library V2.1.0
//
//	Device:
//		STM32Fxx
//
//	Update history
//	---------- ----- -------------------------------------------
//	2013.01.21 v0.0  First cording
//	2012.04.21 v0.1  Commit
//
#ifndef __USBH_WCC_BLUETOOTH_H
#define __USBH_WCC_BLUETOOTH_H

#include <stddef.h>
#include "usbh_core.h"

#define USB_WCC_CLASS			0xe0	// Wireless controller class
#define USB_WCC_CLASS_RF		1		// Sub class / RF controller
#define USB_PROTOCOL_BLUETOOTH	1		// Bluetooth Programing Interface
#define USB_PROTOCOL_BT_AMP		4		// Bluetooth AMP Controller

//
//  Control block table
//
typedef enum {
  IDLE = 0,
  COMMAND_START,
  COMMAND_POST,
  COMMAND_END,
  EVENT_START,
  EVENT_POLL,
  EVENT_POLL_WAIT,
  EVENT_GET,
  EVENT_POST_WAIT,
  EVENT_END,
  ACL_OUT_START,
  ACL_OUT_POST,
  ACL_OUT_WAIT,
  ACL_OUT_END,
  ACL_IN_START,
  ACL_IN_POLL,
  ACL_IN_POLL_WAIT,
  ACL_IN_GET,
  ACL_IN_POST_WAIT,
  ACL_IN_END,
  LAST_STATE
} RUN_STATE;

struct USBH_WCC_Bluetooth_CB {			//
	uint8_t			intrHcNum;			//
	uint8_t			outHcNum;			//
	uint8_t			inHcNum;			//
	uint8_t			intrEp;				//
	uint8_t			outEp;				//
	uint8_t			inEp;				//
	uint16_t		intrEpSize;			//
	uint16_t		outEpSize;			//
	uint16_t		inEpSize;			//
	RUN_STATE		cmdState;			//
	RUN_STATE		evtState;			//
	RUN_STATE		outState;			//
	RUN_STATE		inState;			//
	long			cmdTryCount;		//
	long			evtTryCount;		//
	long			outTryCount;		//
	long			inTryCount;			//
	uint32_t		frame;				//
	size_t			cmdBufLen;			//
	size_t			evtBufLen;			//
	size_t			outBufLen;			//
	size_t			inBufLen;			//
	unsigned char	cmdBuf[64];			// USB endpoint buffer
	unsigned char	evtBuf[16];			//
	unsigned char	outBuf[64];			//
	unsigned char	inBuf[64];			//
};

extern USBH_Class_cb_TypeDef	USBH_WCC_Bluetooth_cb;

#endif  /* __USBH_WCC_BLUETOOTH_H */
